=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2006-12-11 19:23:51 +0000
+++ loggerhead/controllers/__init__.py	2006-12-12 20:34:10 +0000
@@ -25,6 +25,7 @@
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.revision_ui import RevisionUI
+from loggerhead.controllers.inventory_ui import InventoryUI
 
 
 log = logging.getLogger("loggerhead.controllers")
@@ -33,9 +34,8 @@
     changes = ChangeLogUI()
     atom = AtomUI()
     revision = RevisionUI()
+    inventory = InventoryUI()
     
-    @turbogears.expose(template="loggerhead.templates.welcome")
+    @turbogears.expose()
     def index(self):
-        import time
-        log.debug("Happy TurboGears Controller Responding For Duty")
-        return dict(now=time.ctime())
+        raise HTTPRedirect(turbogears.url('/changes'))

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2006-12-12 03:50:13 +0000
+++ loggerhead/controllers/changelog_ui.py	2006-12-12 20:34:10 +0000
@@ -17,6 +17,7 @@
 #
 
 import base64
+import logging
 import os
 import posixpath
 import time
@@ -27,6 +28,8 @@
 from loggerhead.history import History
 from loggerhead import util
 
+log = logging.getLogger("loggerhead.controllers")
+
 
 class ChangeLogUI (object):
 
@@ -38,10 +41,18 @@
         else:
             revid = h.last_revid
 
+        path = kw.get('path', None)
+        
         try:
-            revlist = h.get_short_revision_history_from(revid)
+            if path is not None:
+                inv = h.get_inventory(revid)
+                file_id = inv.path2id(path)
+                revlist = list(h.get_short_revision_history_by_fileid_from(file_id, revid, folder=path.endswith('/')))
+            else:
+                revlist = h.get_short_revision_history_from(revid)
             entries = h.get_changelist(list(revlist)[:20])
-        except:
+        except Exception, x:
+            log.error('Exception fetching changes: %r, %s' % (x, x))
             raise HTTPRedirect(turbogears.url('/changes'))
 
         buttons = [
@@ -59,6 +70,7 @@
             'pagesize': 20,
             'revid': revid,
             'buttons': buttons,
+            'path': path,
         }
         if kw.get('style', None) == 'rss':
             vals['tg_template'] = 'loggerhead.templates.changelog-rss'

=== added file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/inventory_ui.py	2006-12-12 20:34:10 +0000
@@ -0,0 +1,81 @@
+#
+# Copyright (C) 2006  Robey Pointer <robey@lag.net>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import datetime
+import os
+import posixpath
+import textwrap
+
+import turbogears
+from cherrypy import HTTPRedirect, session
+
+from loggerhead.history import History
+from loggerhead import util
+
+
+def dirname(path):
+    while path.endswith('/'):
+        path = path[:-1]
+    path = posixpath.dirname(path)
+    return path
+
+        
+class InventoryUI (object):
+
+    @turbogears.expose(html='loggerhead.templates.inventory')
+    def default(self, *args, **kw):
+        h = History.from_folder(turbogears.config.get('loggerhead.folder'))
+        if len(args) > 0:
+            revid = args[0]
+        else:
+            revid = h.last_revid
+
+        try:
+            rev = h.get_revision(revid)
+            inv = h.get_inventory(revid)
+        except:
+            raise HTTPRedirect(turbogears.url('/changes'))
+        
+        path = kw.get('path', None)
+        if path is None:
+            path = '/'
+            file_id = None
+        else:
+            file_id = inv.path2id(path)
+            
+        buttons = [
+            ('top', turbogears.url('/changes')),
+            ('revision', turbogears.url([ '/revision', revid ])),
+            ('history', turbogears.url([ '/changes', revid ])),
+        ]
+
+        vals = {
+            'branch_name': turbogears.config.get('loggerhead.branch_name'),
+            'util': util,
+            'buttons': buttons,
+            'revid': revid,
+            'change': h.get_change(revid),
+            'path': path,
+            'updir': dirname(path),
+            'filelist': h.get_filelist(inv, path),
+            'history': h,
+            'scan_url': '/revision',
+            'pagesize': 1,
+            'posixpath': posixpath,
+        }
+        return vals

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2006-12-12 03:50:13 +0000
+++ loggerhead/history.py	2006-12-12 20:34:10 +0000
@@ -18,6 +18,7 @@
 
 import cgi
 import datetime
+import posixpath
 import textwrap
 from StringIO import StringIO
 
@@ -31,6 +32,9 @@
 from loggerhead import util
 
 
+
+
+
 class History (object):
     
     def __init__(self):
@@ -80,6 +84,9 @@
         return self._branch.repository.get_revision(revid)
     
     def get_revno(self, revid):
+        if revid not in self._revision_info:
+            # ghost parent?
+            return 'unknown'
         seq, revid, merge_depth, revno_str, end_of_merge = self._revision_info[revid]
         return revno_str
 
@@ -100,6 +107,31 @@
             if len(parents) == 0:
                 return
             revid = self._revision_graph[revid][0]
+    
+    def get_short_revision_history_by_fileid(self, file_id, folder=False):
+        # wow.  is this really the only way we can get this list?  by
+        # man-handling the weave store directly? :-0
+        # FIXME: would be awesome if we could get, for a folder, the list of
+        # revisions where items within that folder changed.
+        w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
+        w_revids = w.versions()
+        revids = [r for r in self._full_history if r in w_revids]
+        return revids
+    
+    def get_short_revision_history_by_fileid_from(self, file_id, revid, folder=False):
+        revids = self.get_short_revision_history_by_fileid(file_id, folder)
+        while True:
+            if revid in revids:
+                yield revid
+            if not self._revision_graph.has_key(revid):
+                return
+            parents = self._revision_graph[revid]
+            if len(parents) == 0:
+                return
+            revid = parents[0]
+
+    def get_inventory(self, revid):
+        return self._branch.repository.get_revision_inventory(revid)
 
     def get_where_merged(self, revid):
         try:
@@ -370,3 +402,28 @@
             removed.append(rich_filename(path, kind))
         
         return util.Container(added=added, renamed=renamed, removed=removed, modified=modified)
+
+    def get_filelist(self, inv, path):
+        """
+        return the list of all files (and their attributes) within a given
+        path subtree.
+        """
+        while path.endswith('/'):
+            path = path[:-1]
+        parity = 0
+        for filepath, entry in inv.entries():
+            if posixpath.dirname(filepath) != path:
+                continue
+            filename = posixpath.basename(filepath)
+            rich_filename = filename
+            if entry.kind == 'directory':
+                rich_filename += '/'
+            
+            # last change:
+            revid = entry.revision
+            
+            yield util.Container(filename=filename, rich_filename=rich_filename, executable=entry.executable, kind=entry.kind,
+                                 revid=revid, revno=self.get_revno(revid), parity=parity)
+            parity ^= 1
+        pass
+

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2006-12-12 02:46:36 +0000
+++ loggerhead/static/css/style.css	2006-12-12 20:34:10 +0000
@@ -23,6 +23,9 @@
 	font-size: 1.6em;
     margin-bottom: 1.5em;
 }
+h1 .filename {
+	font-family: monospace;
+}
 
 .branch-name {
 	color: #47626e;
@@ -205,7 +208,7 @@
 
 /* ----- revision page ----- */
 
-.info-entries {
+.revision-info {
     border: none;
     font-size: smaller;
     margin-bottom: 1em;
@@ -216,12 +219,12 @@
 }
 
 /* secret css-jutsu to make hidden margins vanish on a table */
-.info-entries table {
+.revision-info table {
 	border-collapse: collapse;
     width: 100%;
 }
 
-.info-entries th {
+.revision-info th {
 	background-color: #dee7ec;
     color: black;
 	font-weight: normal;
@@ -234,26 +237,26 @@
 	padding-bottom: 2px;
 }
 
-.info-entries td {
+.revision-info td {
 	font-weight: normal;
   	padding-left: 0.5em;
 	padding-top: 2px;
 	padding-bottom: 2px;
 }
 
-.info-entries table td.date {
-	font-family: monospace;
-}
-
-.info-entries table td.description {
-	font-family: monospace;
-}
-
-.info-entries table td.files {
-	font-family: monospace;
-}
-
-.info-entries table tr.divider {
+.revision-info table td.date {
+	font-family: monospace;
+}
+
+.revision-info table td.description {
+	font-family: monospace;
+}
+
+.revision-info table td.files {
+	font-family: monospace;
+}
+
+.revision-info table tr.divider {
     border-top: 1px solid #8cacbb;
 }
 
@@ -281,6 +284,7 @@
 	padding-left: 1em;
 	padding-top: 1em;
 	border-top: 1px solid #8cacbb;
+	text-align: left;
 }
 
 .diff-chunk {
@@ -290,7 +294,7 @@
 }
 
 .diff-chunk th.lineno {
-	font-weight: normal;
+	font-weight: bold;
     border-bottom: 1px solid #8cacbb;
 }
 .diff-chunk .insert {
@@ -306,12 +310,62 @@
 .diff .lineno {
 	color: purple;
 	background-color: #dee7ec; 
-	width: 3em;
+	width: 4em;
 	text-align: right;
 	padding-right: 3px;
+	border-bottom: 1px solid #8cacbb;
 }
 .diff .info {
 	color: blue;
 	background-color: #dee7ec;
 	margin-top: 15px;
 }
+
+
+/* ----- file inventory ----- */
+
+table.inventory {
+	border-collapse: collapse;
+    width: 80%;
+    font-size: smaller;
+    margin-top: 1em;
+    border: 1px solid #8cacbb;
+}
+
+table.inventory td {
+	padding-top: 3px;
+	padding-bottom: 3px;
+	vertical-align: center;
+}
+
+table.inventory th {
+    background-color: #8cacbb;
+    padding-top: 3px;
+    padding-bottom: 3px;
+	text-align: left;
+}
+table.inventory th.permissions {
+	padding-left: 0.5em;
+}
+
+.parity0 { background-color: #dee7ec; }
+.parity1 { background-color: #eff8fd; }
+
+.inventory td.permissions {
+    font-family: monospace;
+    width: 10em;
+	padding-left: 0.5em;
+}
+
+.inventory td.filename {
+    font-family: monospace;
+}
+
+.inventory td.revision {
+    width: 8em;
+}
+
+.inventory td.changes-link {
+    width: 6em;
+}
+

=== modified file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2006-12-12 03:50:13 +0000
+++ loggerhead/templates/changelog.kid	2006-12-12 20:34:10 +0000
@@ -15,8 +15,10 @@
 
 ${navbar()}
 
-<h1> <span class="branch-name">${branch_name}</span> : changes </h1>
-    
+<h1> <span class="branch-name">${branch_name}</span> : changes
+<span py:if="path"> to <span class="filename">${path}</span></span>
+</h1>
+
 <div class="log-entries">
     <div py:for="entry in changes" class="revision">
         <div class="revision-header">

=== added file 'loggerhead/templates/inventory.kid'
--- loggerhead/templates/inventory.kid	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/inventory.kid	2006-12-12 20:34:10 +0000
@@ -0,0 +1,75 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#"
+    py:extends="'master.kid'">
+<head>
+    <meta content="text/html; charset=utf-8" http-equiv="Content-Type" py:replace="''"/>
+    <title> ${branch_name} : files for revision ${change.revno} </title>
+</head>
+
+<body>
+
+${navbar()}
+
+<h1> <span class="branch-name">${branch_name}</span> : files for revision ${change.revno} </h1>
+
+<!-- !FIXME: this is just copied verbatim from revision.kid -->
+<div class="revision-info">
+    <table>
+        <tr>
+            <th class="author">committed by:</th>
+            <td class="author"> ${util.hide_email(change.author)} </td>
+        </tr>
+        <tr>
+            <th class="date">date:</th>
+            <td class="date"> ${change.date.strftime('%d %b %Y %H:%M')} </td>
+        </tr>
+
+        <tr py:if="len(change.merge_points) > 0">
+            <th class="children"> merged in: </th>
+            <td class="children">
+                <span py:for="child in change.merge_points"> ${revlink(child.revid, '(' + child.revno + ')')} &nbsp; </span>
+            </td>
+        </tr>
+        <tr py:if="len(change.parents) > 1">
+        	<th class="parents"> merged from: </th>
+        	<td class="parents">
+        	    <span py:for="parent in change.parents"><span py:if="parent.revid != change.parents[0].revid"> ${revlink(parent.revid, '(' + parent.revno + ')')} &nbsp; </span></span>
+        	</td>
+        </tr>
+
+        <tr>
+            <th class="description">description:</th>
+            <td class="description"><span py:for="line in change.comment_clean">${XML(line)} <br /></span> </td>
+        </tr>
+    </table>
+</div>
+
+folder: <span class="folder"> ${path} </span>
+
+<table class="inventory" width="100%">
+    <tr class="header">
+        <th class="permissions"> Permissions </th>
+        <th> Filename </th>
+        <th> Last change </th>
+        <th> History </th>
+    </tr>
+    
+    <tr class="parity1" py:if="updir">
+        <td class="permissions">drwxr-xr-x</td>
+        <td class="filename directory"><a href="${tg.url([ '/inventory', revid ], path=updir)}"> (up) </a></td>
+        <td> </td> <td> </td>
+    </tr>
+
+    <tr py:for="file in filelist" class="parity${file.parity}">
+        <td class="permissions"> ${util.fake_permissions(file.kind, file.executable)} </td>
+        <td class="filename ${file.kind}">
+            ${file.filename}<span py:if="file.kind == 'symlink'">@</span><span py:if="file.kind == 'directory'">/</span>
+        </td>
+        <td class="revision"> ${revlink(file.revid, file.revno)} </td>
+        <td class="changes-link"> <a href="${tg.url([ '/changes', file.revid ], path=posixpath.join(path, file.rich_filename))}"> (changes) </a></td>
+    </tr>
+    <!-- #entries# -->
+</table>
+
+</body>
+</html>

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2006-12-12 03:50:13 +0000
+++ loggerhead/templates/revision.kid	2006-12-12 20:34:10 +0000
@@ -12,7 +12,7 @@
 
 <h1> <span class="branch-name">${branch_name}</span> : revision ${change.revno} </h1>
 
-<div class="info-entries">
+<div class="revision-info">
     <table>
         <tr>
             <th class="author">committed by:</th>

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2006-12-12 00:52:31 +0000
+++ loggerhead/util.py	2006-12-12 20:34:10 +0000
@@ -145,4 +145,15 @@
     in displaying monospace text.
     """
     s = cgi.escape(s.expandtabs()).replace(' ', '&nbsp;')
-    return s;
+    return s
+
+
+def fake_permissions(kind, executable):
+    # fake up unix-style permissions given only a "kind" and executable bit
+    if kind == 'directory':
+        return 'drwxr-xr-x'
+    if executable:
+        return '-rwxr-xr-x'
+    return '-rw-r--r--'
+
+        

